VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsModules"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'---------------------------------------------------------------------------------------
' Module    : clsModules
' DateTime  : 06/02/2007 13:02
' Author    : Vb System Library
' Copyright : (C) 2007 L'quipe Vb System Library
'              See "modPresentation" for more information
' Purpose   : Grer les modules des processus
'---------------------------------------------------------------------------------------

Option Explicit


' Infos d'un module
Public Type ModuleInfos
    ModuleID            As Long     ' ID du module
    ProcessID           As Long     ' ID du processus qui utilise le module
    GlobalcountUsage    As Long     ' compteur d'usage global pour toutes les instances prsentes dans la mmoire du systme
    ProcesscountUsage   As Long     ' commteur d'usage du module pour le processus
    BaseAddr            As Long     ' adresse de dbut du module en mmoire
    BaseSize            As Long     ' taille du module en mmoire
    hModule             As Long     ' HMODULE du module
    ModuleName          As String   ' nom du module
    ExeFile             As String   ' path du module
End Type


'---------------------------------------------------------------------------------------
' Procedure : CreateModuleList
' DateTime  : 06/02/2007 13:13
' Author    : Vb System Library
' Purpose   : Cr une liste des modules d'un processus
' Params    : PID : ID du processus
'               mdList() : Tableau des ID des modules d'un processus
'---------------------------------------------------------------------------------------
Public Sub CreateModuleList(ByVal PID As Long, ByRef mdList() As ModuleInfos)
    Dim lSnap As Long
    Dim X As Long
    Dim mdMOD As MODULEENTRY32
    Dim mdTemp() As ModuleInfos
    X = 0

    'cration du snapshot des modules
    lSnap = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE, PID)

    mdMOD.dwSize = Len(mdMOD)
    
    's'occupe du premier module rencontr dans le snap
    If Module32First(lSnap, mdMOD) Then
        'alors on a trouv un module, on va pouvoir continuer plus loin
        
        'on dimensionne notre tableau
        ReDim mdTemp(0)
        'on stocke les infos du premier module dans la liste,  l'emplacement 0
        With mdTemp(0)
            .BaseAddr = mdMOD.modBaseAddr
            .BaseSize = mdMOD.modBaseSize
            .ExeFile = mdMOD.szExeFile
            .GlobalcountUsage = mdMOD.GlblcntUsage
            .hModule = mdMOD.hModule
            .ModuleID = mdMOD.th32ModuleID
            .ModuleName = mdMOD.szModule
            .ProcesscountUsage = mdMOD.ProccntUsage
            .ProcessID = mdMOD.th32ProcessID
        End With
        mdMOD.dwSize = Len(mdMOD)
        
        'continue tant qu'il y a des nouveaux modules dans le snapshot
        Do While Module32Next(lSnap, mdMOD)

            'on redimensionne le tableau de 1 plus grand, pour pouvoir acceullir la liste temporaire du module en cours
            ReDim Preserve mdTemp(X)

            'stocke les infos du module en cours  l'emplacement x
            With mdTemp(X)
                .BaseAddr = mdMOD.modBaseAddr
                .BaseSize = mdMOD.modBaseSize
                .ExeFile = mdMOD.szExeFile
                If InStr(.ExeFile, vbNullChar) Then .ExeFile = Left$(.ExeFile, InStr(.ExeFile, vbNullChar) - 1)
                .ExeFile = Trim$(.ExeFile)
                .GlobalcountUsage = mdMOD.GlblcntUsage
                .hModule = mdMOD.hModule
                .ModuleID = mdMOD.th32ModuleID
                .ModuleName = mdMOD.szModule
                If InStr(.ModuleName, vbNullChar) Then .ModuleName = Left$(.ModuleName, InStr(.ModuleName, vbNullChar) - 1)
                .ModuleName = Trim$(.ModuleName)
                .ProcesscountUsage = mdMOD.ProccntUsage
                .ProcessID = mdMOD.th32ProcessID
            End With

            'prpare la taille pour le prochain module
            mdMOD.dwSize = Len(mdMOD)
            X = X + 1
        Loop
    Else
        ReDim mdTemp(1)
    End If
    
    ReDim Preserve mdTemp(UBound(mdTemp()) - 1)
    'stockage de la liste des modules
    mdList = mdTemp 'on aurait directement pu travailler sur mdList sans passer par mdTemp, mais il est plus propre de ne changer mdList (d'ailleurs toute autre valeur renvoye par une sub/fonction) uniquement A LA FIN de cette sub/fonction
    
    'on le handle du snap
    CloseHandle lSnap
End Sub


'---------------------------------------------------------------------------------------
' Procedure : LoadModuleInProcess
' DateTime  : 06/02/2007 13:02
' Author    : Vb System Library
' Purpose   : Charge un module dans un processus (injecter)
' Params    : ProcessId : ID du processus vis
'               ModulePathName : Chemin d'accs au module (fichier DLL)
'---------------------------------------------------------------------------------------
Public Function LoadModuleInProcess(ByVal ProcessID As Long, ByVal ModulePathName As String) As Long
        
    Dim hKernel32   As Long
    Dim hThread     As Long
    Dim hVirtual    As Long
    Dim hProcess    As Long
    Dim hFunc       As Long
    
    'Ouverture du process
    hProcess = OpenProcess(PROCESS_CREATE_THREAD Or PROCESS_VM_OPERATION Or PROCESS_VM_WRITE Or PROCESS_VM_READ, False, ProcessID)
    
    If hProcess Then
        'On recupere le Handle de Kernel32 qui contient la fontion LoadLibrary
        hKernel32 = GetModuleHandle(KRN_DLL)
        
        'On recupere l'adresse de la fonction LoadLibrary
        hFunc = GetProcAddress(hKernel32, KRN_LOAD)
        
        'On alloue une zone memoire dans le process de destination
        hVirtual = VirtualAllocEx(hProcess, ByVal 0&, Len(ModulePathName), MEM_COMMIT, PAGE_READWRITE)
        
        'On ecrit le chemin du module dans la zone memoire que l'on vient d'allouer
        If WriteProcessMemory(hProcess, ByVal hVirtual, ByVal ModulePathName, Len(ModulePathName), ByVal 0&) Then
            
            'Creation d'un Thread dans le Process,
            'ce Thread va faire appel a LoadLibrary avec pour parametre hVirtual
            'qui pointe vers la zone contenant le nom du module a charger
            hThread = CreateRemoteThread(hProcess, ByVal 0&, 0, ByVal hFunc, ByVal hVirtual, 0, ByVal 0&)
            If hThread Then
                'Maitenant on attend la fin du Thread
                'Ici on attend 2,5 secondes ce qui est plus que suffisant pour
                'charger un module
                WaitForSingleObject hThread, &H9C4
                'On recupere code de sortie du Thread qui est le code de
                'retour de la fonction LoadLibrary et qui contient le Handle
                'du module charg si tout s'est bien pass
                GetExitCodeThread hThread, LoadModuleInProcess
                'On ferme le Handle du Thread
                CloseHandle hThread
            End If
            
        End If
        
        'On libere la zone alloue dans le Process
        VirtualFreeEx hProcess, hVirtual, Len(ModulePathName), MEM_RELEASE
          
        'On ferme le Handle du Process
        CloseHandle hProcess
    End If
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : UnLoadModuleFromProcess
' DateTime  : 06/02/2007 13:04
' Author    : Vb System Library
' Purpose   : Dcharge un module du processus vis
' Params    : ProcessId : ID du processus vis
'               Pour dcharger le module, on peut donner
'               - Le chemin d'accs  la DLL : ModulePathName
'               - Le handle du module charg en mmoire
'---------------------------------------------------------------------------------------
Public Function UnLoadModuleFromProcess(ByVal ProcessID As Long, Optional ByVal ModulePathName As String = "", Optional ByVal hMod As Long = 0) As Long
    
    Dim hKernel32   As Long
    Dim hThread     As Long
    Dim hProcess    As Long
    Dim hFunc       As Long
    Dim hVirtual    As Long
    
    hProcess = OpenProcess(PROCESS_CREATE_THREAD Or PROCESS_VM_OPERATION Or PROCESS_VM_WRITE Or PROCESS_VM_READ, False, ProcessID)
  
    If hProcess Then
        hKernel32 = GetModuleHandle(KRN_DLL)
        hFunc = GetProcAddress(hKernel32, GET_HMOD)
        
        ' Si on a pas le handle du module on le recupere en faisant appel
        ' a GetModuleHandle depuis le process cible
        If Not hMod Then
            hVirtual = VirtualAllocEx(hProcess, ByVal 0&, Len(ModulePathName), MEM_COMMIT, PAGE_READWRITE)
            If WriteProcessMemory(hProcess, ByVal hVirtual, ByVal ModulePathName, Len(ModulePathName), ByVal 0&) Then
                hThread = CreateRemoteThread(hProcess, ByVal 0&, 0, ByVal hFunc, ByVal hVirtual, 0, ByVal 0&)
                If hThread Then
                    WaitForSingleObject hThread, &H9C4
                    'on recupere le handle du Module dans hMod
                    GetExitCodeThread hThread, hMod
                    CloseHandle hThread
                End If
            End If
        End If
        
        'Si on le handle
        If hMod Then
            hFunc = GetProcAddress(hKernel32, KRN_FREE)
            hThread = CreateRemoteThread(hProcess, ByVal 0&, 0, ByVal hFunc, ByVal hMod, 0, ByVal 0&)
            If hThread Then
                WaitForSingleObject hThread, &H9C4
                'on recupere le code de retour de FreeLibrary (si = 1 le module a bien t decharg)
                GetExitCodeThread hThread, UnLoadModuleFromProcess
                CloseHandle hThread
            End If
        End If
        
        CloseHandle hProcess
    End If
    
End Function
