VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Window"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'---------------------------------------------------------------------------------------
' Module    : Window
' DateTime  : 06/12/2006 20:56
' Author    : Vb System Library
' Copyright : (C) 2007 L'quipe Vb System Library
'              See "modPresentation" for more information
' Purpose   : Objet fenetre
'---------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------
'
' Proprits
'       [statiques]
'   Handle
'   ParentProcessID
'   ParentThreadID
'       [dynamiques]
'   Caption
'   Left
'   Top
'   Width
'   Height
'   Visible
'   Enabled
'
' Mthodes
'   GetWindowByHandle
'   RefreshInfos
'   CloseWindow
'
'---------------------------------------------------------------------------------------

Option Explicit

Private clsWindow As New clsWindows

' Les infos sur cette fenetre
Private tabInfos(0 To 9) As Variant

' Les diffrentes infos de la fenetre
Private Enum enumWindowInfo
    ewindowHandle = 0                               ' Le handle de la fenetre
    ewindowCaption = 1                              ' Le titre de la fenetre
    ewindowLeft = 2                                 ' Position
    ewindowTop = 3                                  ' Position
    ewindowWidth = 4                                ' Position
    ewindowHeight = 5                               ' Position
    ewindowVisible = 6                              ' Fenetre visible ou pas
    ewindowEnabled = 7                              ' Active
    ewindowParentProcessID = 8                      ' Processus crateur de la fenetre
    ewindowParentThreadID = 9                       ' Thread crateur de la fenetre
End Enum


'---------------------------------------------------------------------------------------
' Les mthodes
'---------------------------------------------------------------------------------------


' Retrouve les infos statiques fenetre  partir de son handle
' [Optionnel] RefreshInfos : Active ou non le rafraichissement des infos dynamiques du processus
Public Sub GetWindowByHandle(ByVal Handle As Long, Optional ByVal bRefreshInfos As Boolean = False)
    tabInfos(ewindowHandle) = Handle
    ' Les informations statiques
        ' Processus parent
    tabInfos(ewindowParentProcessID) = clsWindow.GetWindowParentProcess(Handle)
        ' Thread parent
    tabInfos(ewindowParentThreadID) = clsWindow.GetWindowParentThreadID(Handle)
    ' Les informations dynamiques
    If bRefreshInfos = True Then RefreshInfos
End Sub


' Rafraichit les informations dynamiques
Public Sub RefreshInfos()
    Dim hWnd As Long
    Dim tmpRect As RECT
    hWnd = tabInfos(ewindowHandle)
    ' Le titre de la fenetre
    tabInfos(ewindowCaption) = clsWindow.GetWindowCaption(hWnd)
    ' La position de la fenetre
    tmpRect = clsWindow.GetWindowPosition(hWnd)
    tabInfos(ewindowLeft) = tmpRect.Left
    tabInfos(ewindowTop) = tmpRect.Top
    tabInfos(ewindowWidth) = tmpRect.Right - tmpRect.Left
    tabInfos(ewindowHeight) = tmpRect.Bottom - tmpRect.Top
    ' Visibilit de la fenetre
    tabInfos(ewindowVisible) = clsWindow.WindowVisible(hWnd)
    ' Active
    tabInfos(ewindowEnabled) = clsWindow.WindowEnabled(hWnd)
End Sub


' Ferme la fenetre
' (Note : Close est un nom rserv, on ne peut donc pas appeler cette mthode "Close")
Public Sub CloseWindow()
    clsWindow.CloseWindow Val(tabInfos(ewindowHandle))
End Sub


'---------------------------------------------------------------------------------------
' Les proprits
'---------------------------------------------------------------------------------------

Public Property Get Handle() As Long
    Handle = tabInfos(ewindowHandle)
End Property

Public Property Get Caption() As String
    Caption = tabInfos(ewindowCaption)
End Property
Public Property Let Caption(ByVal sCaption As String)
    clsWindow.SetWindowCaption tabInfos(ewindowHandle), sCaption
    tabInfos(ewindowCaption) = clsWindow.GetWindowCaption(tabInfos(ewindowHandle))
End Property

Public Property Get Left() As Long
    Left = tabInfos(ewindowLeft)
End Property
Public Property Let Left(ByVal lLeft As Long)
    Dim tmpRect As RECT
    clsWindow.SetWindowPosition tabInfos(ewindowHandle), lLeft
    tmpRect = clsWindow.GetWindowPosition(tabInfos(ewindowHandle))
    tabInfos(ewindowLeft) = tmpRect.Left
End Property

Public Property Get Top() As Long
    Top = tabInfos(ewindowTop)
End Property
Public Property Let Top(ByVal lTop As Long)
    Dim tmpRect As RECT
    clsWindow.SetWindowPosition tabInfos(ewindowHandle), , lTop
    tmpRect = clsWindow.GetWindowPosition(tabInfos(ewindowHandle))
    tabInfos(ewindowTop) = tmpRect.Top
End Property

Public Property Get Width() As Long
    Width = tabInfos(ewindowWidth)
End Property
Public Property Let Width(ByVal lWidth As Long)
    Dim tmpRect As RECT
    clsWindow.SetWindowPosition tabInfos(ewindowHandle), , , lWidth
    tmpRect = clsWindow.GetWindowPosition(tabInfos(ewindowHandle))
    tabInfos(ewindowWidth) = tmpRect.Right - tmpRect.Left
End Property

Public Property Get Height() As Long
    Height = tabInfos(ewindowHeight)
End Property
Public Property Let Height(ByVal lHeight As Long)
    Dim tmpRect As RECT
    clsWindow.SetWindowPosition tabInfos(ewindowHandle), , , , lHeight
    tmpRect = clsWindow.GetWindowPosition(tabInfos(ewindowHandle))
    tabInfos(ewindowHeight) = tmpRect.Bottom - tmpRect.Top
End Property

Public Property Get Visible() As Boolean
    Visible = tabInfos(ewindowVisible)
End Property
Public Property Let Visible(ByVal NewValue As Boolean)
    If NewValue = True Then
        clsWindow.ShowWindow tabInfos(ewindowHandle)
    Else
        clsWindow.HideWindow tabInfos(ewindowHandle)
    End If
    tabInfos(ewindowVisible) = clsWindow.WindowVisible(tabInfos(ewindowHandle))
End Property

Public Property Get Enabled() As Boolean
    Enabled = tabInfos(ewindowEnabled)
End Property
Public Property Let Enabled(ByVal NewValue As Boolean)
    If NewValue = True Then
        clsWindow.EnableWindow tabInfos(ewindowHandle)
    Else
        clsWindow.DisableWindow tabInfos(ewindowHandle)
    End If
    tabInfos(ewindowEnabled) = clsWindow.WindowEnabled(tabInfos(ewindowHandle))
End Property

Public Property Get ParentProcessID() As Long
    ParentProcessID = tabInfos(ewindowParentProcessID)
End Property

Public Property Get ParentThreadID() As Long
    ParentThreadID = tabInfos(ewindowParentThreadID)
End Property
